#
#  Installer for PCC II / PlayVCR
#
#  Original version by Christian Brckner
#  This version by Stefan Reuther
#

SetCompressor   /SOLID lzma

!include "FileFunc.nsh"
!insertmacro GetDrives
!insertmacro GetSize
!insertmacro un.GetSize

Name            "PlayVCR"
OutFile         "PlayVCR-Installer.exe"
Caption         "PlayVCR"
InstallDir      $PROGRAMFILES\PlayVCR

XPStyle         on
InstProgressFlags smooth
InstallDirRegKey HKEY_LOCAL_MACHINE "Software\Phost\PlayVCR" ""

Var     winplandir
Var     mytemp

# Pages for Installer
LicenseBkColor /windows
PageEx license
  Caption ": Welcome"
  LicenseText "This will install PlayVCR. Please close all your Planets tools, especially Winplan, before proceeding." "Next >"
  LicenseData readme.rtf
PageExEnd

PageEx components
  PageCallbacks componentsInit "" ""
PageExEnd

PageEx directory
  # This page prompts the user for the target directory
PageExEnd

PageEx directory
  DirText "Enter the directory where you have Winplan installed. Leave blank to install without Winplan integration."
  DirVar $winplandir
  DirVerify leave
  PageCallbacks winplanPre "" winplanPost
PageExEnd

InstallColors /windows
PageEx instfiles
PageExEnd



# Pages for Uninstaller
PageEx un.uninstConfirm
  UninstallText "This will uninstall (remove) PlayVCR from your computer. Please close all your Planets tools, especially Winplan and PlayVCR, before proceeding."
PageExEnd

PageEx un.instfiles
PageExEnd


Section         "Program Files (mandatory)"
  SetOutPath        $INSTDIR
  File /r /x *.lang playvcr\*.*
  Call              writeResourceConfig
  Call              writeWinplanIntegration
  WriteUninstaller $INSTDIR\uninstaller.exe

  # Registry stuff
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Phost\PlayVCR" "" "$INSTDIR"
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Phost\PlayVCR" "winplandir" "$winplandir"

  StrCpy $0 "Software\Microsoft\Windows\CurrentVersion\Uninstall\PlayVCR"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "DisplayName" "PlayVCR (remove only)"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "UninstallString" '"$INSTDIR\uninstaller.exe"'
  WriteRegStr HKEY_LOCAL_MACHINE $0 "DisplayIcon" "$INSTDIR\resource\playvcr.bmp"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "InstallLocation" "$INSTDIR"
  WriteRegDWORD HKEY_LOCAL_MACHINE $0 "NoModify" 1
  WriteRegDWORD HKEY_LOCAL_MACHINE $0 "NoRepair" 1

  # File Type
  WriteRegStr HKEY_CLASSES_ROOT ".ccb" "" "PlayVCR.Simulation"

  WriteRegStr HKEY_CLASSES_ROOT "PlayVCR.Simulation" "" "PlayVCR Saved Simulation Setup"
  # WriteRegStr HKEY_CLASSES_ROOT "PlayVCR.Simulation\DefaultIcon" "" "$INSTDIR\resource\playvcr.bmp"
  WriteRegStr HKEY_CLASSES_ROOT "PlayVCR.Simulation\shell\open\command" "" '"$INSTDIR\playvcr.exe" %1'
SectionEnd

# Section               "Winplan Integration"
# SectionEnd

Section         "Language Support"
  SetOutPath        $INSTDIR\resource
  File              playvcr\resource\*.lang
SectionEnd

Section         "Create Start Menu Entries"
  # This SetOutPath sets the working directory stored in the shortcuts
  SetOutPath        $INSTDIR
  CreateDirectory   "$SMPROGRAMS\PlayVCR"
  StrCpy $0 ""
  StrCmp $winplandir "" noArgs
  StrCpy $0 '-dir "$winplandir"'
 noArgs:
  CreateShortCut    "$SMPROGRAMS\PlayVCR\PlayVCR.lnk" "$INSTDIR\playvcr.exe" $0 "$INSTDIR\resource\playvcr.bmp"
  CreateShortCut    "$SMPROGRAMS\PlayVCR\Uninstall PlayVCR.lnk" "$INSTDIR\uninstaller.exe"
SectionEnd


Section         Uninstall
  # First, uninstall Winplan integration
  ClearErrors
  ReadRegStr $winplandir HKEY_LOCAL_MACHINE "Software\Phost\PlayVCR" "winplandir"
  IfErrors skipUninstallIntegrator
  StrCmp $winplandir "" skipUninstallIntegrator

  # Is winplan.exe actually our program?
  IfFileExists $winplandir\winplano.exe "" skipUninstallIntegrator
  IfFileExists $winplandir\winplan.exe "" skipUninstallIntegrator
  ${un.GetSize} $winplandir "/M=winplan.exe /S=0B /G=0" $0 $1 $2
  IntCmp $0 500000 isWinplan isIntegrator isWinplan
 isIntegrator:
  Delete /REBOOTOK $winplandir\winplan.exe
  Rename /REBOOTOK $winplandir\winplano.exe $winplandir\winplan.exe
  Delete $winplandir\vcr.cfg

 isWinplan:
 skipUninstallIntegrator:
  ClearErrors

  RMDir /r      $INSTDIR
  DeleteRegKey HKEY_LOCAL_MACHINE "SOFTWARE\Phost\PlayVCR"
  DeleteRegKey HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\PlayVCR"

  # File Type
  DeleteRegKey HKEY_CLASSES_ROOT "PlayVCR.Simulation"
  ReadRegStr $0 HKEY_CLASSES_ROOT ".ccb" ""
  StrCmp $0 "PlayVCR.Simulation" "" skipUninstallAssociation
  DeleteRegKey HKEY_CLASSES_ROOT ".ccb"
 skipUninstallAssociation:

  # Start Menu
  RMDir /r      "$SMPROGRAMS\PlayVCR"
SectionEnd

# Functions

Function writeResourceConfig
  StrCpy $0 ""
  StrCmp $winplandir "" noWinplanDir

  IfFileExists $winplandir\bmp "" noBmp
  StrCpy $0 "$0wp:$winplandir\bmp$\r$\n"
 noBmp:

  IfFileExists $winplandir\wpvcr.dll "" noWPVCR
  StrCpy $0 "$0wpvcr:$winplandir\wpvcr.dll$\r$\n"
 noWPVCR:

  IfFileExists $winplandir\cc256.res "" noCC256
  StrCpy $0 "$0res:$winplandir\cc256.res$\r$\n"
 noCC256:
 noWinplanDir:

  StrCmp $0 "" noCCRES
  FileOpen $1 "$INSTDIR\resource\cc-res.cfg" "w"
  DetailPrint "Writing $INSTDIR\resource\cc-res.cfg..."
  FileWrite $1 $0
  FileClose $1
 noCCRES:
FunctionEnd

Function writeWinplanIntegration
  StrCmp $winplandir "" noWinplan
  IfFileExists $winplandir\winplan.exe "" noWinplan

  # Check whether there already is a Winplan Integrator installed, to
  # avoid that we overwrite a user's real Winplan
  ${GetSize} $winplandir "/M=winplan.exe /S=0B /G=0" $0 $1 $2
  IntCmp $0 500000 isWinplan isIntegrator isWinplan
 isWinplan:
  Delete $winplandir\winplano.exe
  Rename $winplandir\winplan.exe $winplandir\winplano.exe
 isIntegrator:
  File "/oname=$winplandir\winplan.exe" vcrwrap.exe
  FileOpen $1 "$winplandir\vcr.cfg" "w"
  DetailPrint "Writing $winplandir\vcr.cfg..."
  FileWrite $1 "playvcr=$INSTDIR\playvcr.exe"
  FileClose $1

 noWinplan:
FunctionEnd


# Look for Winplan, recursively.
# in:  $winplandir -- directory to scan
# out: $winplandir -- "" if Winplan not found, Winplan directory otherwise
Function lookForWinplan
  IfFileExists $winplandir\winplan.exe found

  # We try all combinations of "games", "plan1", "spiele", "planets", and "winplan"
  # that exist on the user's drive.
  IfFileExists $winplandir\games "" notGames
  Push $winplandir
  StrCpy $winplandir $winplandir\games
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notGames:
  IfFileExists $winplandir\plan1 "" notPlan1
  Push $winplandir
  StrCpy $winplandir $winplandir\plan1
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notPlan1:
  IfFileExists $winplandir\spiele "" notSpiele
  Push $winplandir
  StrCpy $winplandir $winplandir\spiele
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notSpiele:
  IfFileExists $winplandir\planets "" notPlanets
  Push $winplandir
  StrCpy $winplandir $winplandir\planets
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notPlanets:
  IfFileExists $winplandir\winplan "" notWinplan
  Push $winplandir
  StrCpy $winplandir $winplandir\winplan
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notWinplan:
  StrCpy $winplandir ""
  Goto found

 foundPop:
  Pop $mytemp
 found:
FunctionEnd

# Find Winplan on a drive. Used as callback by $GetDrives
# in: $9 -- drive letter ("c:\"), $8 -- drive type
Function lookForWinplanOnDrive
  StrCpy $winplandir "$9"

  # remove trailing \ if there is one:
  StrCpy $mytemp $winplandir "" -1
  StrCmp $mytemp \ "" notSlash
  StrCpy $winplandir $winplandir -1
 notSlash:

  # look for Winplan in that directory
  Call lookForWinplan
  StrCmp $winplandir "" "" notFound
  Push ""
  Goto return
 notFound:
  Push "StopGetDrives"
 return:
FunctionEnd

# Pre function for Winplan directory page
Function winplanPre
  StrCpy $winplandir ""
  ${GetDrives} "HDD" lookForWinplanOnDrive
FunctionEnd

# Post function for Winplan directory page
Function winplanPost
  StrCmp $winplandir "" found
  IfFileExists $winplandir\winplan.exe found
  MessageBox MB_YESNO "winplan.exe not found in specified directory. Do you want to install without Winplan integration? (Click $\"No$\" to enter a different directory.)" \
    IDYES installYes IDNO installNo

 installNo:
  Abort

 installYes:
  StrCpy $winplandir ""
 found:
FunctionEnd

Function componentsInit
  SectionSetFlags 0 17 # means not unselectable
FunctionEnd

Function un.onUninstSuccess
  IfRebootFlag "" noReboot
  MessageBox MB_ICONSTOP|MB_YESNO "The computer needs to be rebooted to complete the uninstallation. Reboot now?" IDNO noReboot
  Reboot
 noReboot:
FunctionEnd

